import os
import re
import sys
import argparse
from . import get_data
from .utils import (
    organise_media,
    download_media,
    match_snap_id
)


def main():
    USAGE_MSG = 'snapmap_archiver -o [OUTPUT DIR] -l="[LATITUDE],[LONGITUDE]' \
                ' -l="[LATITUDE],[LONGITUDE] [SNAP URL (optional)]"'
    geo_msg = 'Use comma seperated values for ' \
        'latitude/longitude, e.g: -l="35.0,67.0"'
    parser = argparse.ArgumentParser(
        description='Download content from Snapmaps', usage=USAGE_MSG)
    parser.add_argument(
        '-o', dest='output_dir', type=str,
        help='Output directory for downloaded content.')
    parser.add_argument(
        '-l', '--location', dest='location',
        type=str, help='Latitude/longitude of desired area. Can be used multiple times',
        action='append', nargs='*')
    parser.add_argument(
        '-z', dest='zoom_depth', type=float,
        help='Snapmaps zoom depth, default is 5.')
    parser.add_argument(
        '-r', dest='radius', type=int,
        help='Maximum Snap radius in meters, default is 10000.')
    parser.add_argument(
        '--file', dest='input_file', type=str,
        help='File containing line-separated Snap URLs or IDs'
    )
    parser.add_argument(
        '--write-json', dest='write_json', action='store_true',
        default=False, help='Write Snap metadata JSON.')
    parser.add_argument(
        '--no-overlay', dest='no_overlay', action='store_true',
        default=False,
        help='Do not use ffmpeg to merge graphical '
             'elements to video Snaps. Default is False')
    args, unknown = parser.parse_known_args()

    snap_ids = []
    if unknown:
        snap_ids = [match_snap_id(i) for i in unknown if re.match(
            r'https?:\/\/map\.snapchat\.com\/ttp\/snap\/W7_(?:[aA-zZ0-9\-_\+]{22})(?:[aA-zZ0-9-_\+]{28})AAAAAA\/?(?:@-?[0-9]{1,3}\.?[0-9]{0,},-?[0-9]{1,3}\.?[0-9]{0,}(?:,[0-9]{1,3}\.?[0-9]{0,}z))?',
            i
        )]

    if args.input_file:
        if os.path.isfile(args.input_file):
            with open(args.input_file, 'r') as f:
                to_format = f.read().split('\n')
                for candidate in to_format:
                    if len(candidate) > 0:
                        try:
                            snap_ids.append(match_snap_id(candidate))
                        except Exception:
                            print(
                                f'{candidate} is not a valid Snap URL or ID.')
        else:
            sys.exit('Input file does not exist.')

    if not args.output_dir:
        print('Output directory (-o) is required.')
        sys.exit(USAGE_MSG)

    if not args.location and not snap_ids and not args.input_file:
        print('No form of input was supplied.')
        sys.exit(USAGE_MSG)

    if args.location:
        for coords in args.location:
            if ',' not in coords[0]:
                sys.exit(f'{geo_msg}\n{coords} is not valid.')

    if not os.path.isdir(args.output_dir):
        try:
            os.mkdir(args.output_dir)
        except PermissionError:
            sys.exit(f'Could not create directory "{args.output_dir}"')

    if not args.radius:
        args.radius = 10000
    elif args.radius > 85000:
        print('Supplied radius value is too large '
              '(above 85,000). Defaulting to 85000.')
        args.radius = 85000

    if args.location:
        for coords in args.location:
            try:
                geo_data = coords[0].split(',', 1)
            except Exception:
                sys.exit(geo_msg)
            api_response = get_data.api_query(
                float(geo_data[0]), float(geo_data[1]), max_radius=args.radius)
            download_media(
                args.output_dir,
                organise_media(api_response),
                args.write_json,
                args.no_overlay
            )

    if snap_ids:
        download_media(
            args.output_dir,
            organise_media(
                get_data.api_query(
                    snap_ids=snap_ids, mode='snap')['elements']),
            args.write_json,
            args.no_overlay
        )



import sys
import json
import subprocess
import os
import re


def match_snap_id(url: str) -> str:
    return re.search(
        r'(W7_(?:[aA-zZ0-9\-_\+]{22})(?:[aA-zZ0-9-_\+]{28})AAAAAA)',
        url).group(1)


def organise_media(api_data):
    to_download = []
    print(f'Found {len(api_data)} Snaps.')
    for entry in api_data:
        data_dict = {
            'id': entry['id'],
            'create_time': entry['timestamp'],
            'media': {}
        }
        try:
            for locale in entry['snapInfo']['title']['strings']:
                if locale['locale'] == 'en':
                    data_dict['location'] = locale['text']
        except KeyError:
            data_dict['location'] = \
                entry['snapInfo']['localitySubtitle']['fallback']
        try:
            data_dict['media']['overlayText'] = \
                entry['snapInfo']['overlayText']
        except KeyError:
            data_dict['media']['overlayText'] = None
        try:
            data_dict['media']['raw_url'] = \
                entry['snapInfo']['streamingMediaInfo']['prefixUrl'] + 'media.mp4'
            data_dict['media']['filetype'] = "mp4"
            try:
                data_dict['media']['video_overlay'] = \
                    entry['snapInfo']['streamingMediaInfo']['prefixUrl'] + \
                    entry['snapInfo']['streamingMediaInfo']['overlayUrl']
            except KeyError:
                data_dict['media']['video_overlay'] = None
        except KeyError:
            try:
                data_dict['media']['raw_url'] = \
                    entry['snapInfo']['publicMediaInfo']\
                         ['publicImage''MediaInfo']['mediaUrl']
                data_dict['media']['filetype'] = "jpg"
            except KeyError:
                for i in entry['snapInfo'].items():
                    if i[0] == 'streamingThumbnailInfo':  # For some reason JSON throws an error if you just query this key directly, so you have to do it this way.
                        data_dict['media']['raw_url'] = \
                            i[1]['infos'][-1]['thumbnailUrl']
                if len(data_dict['media']) == 0:
                    continue  # If there's no video file and no video/image thumbnail, just skip the snap since there's nothing to download
        to_download.append(data_dict)
    return(to_download)


def download_media(
        output_dir, organised_data, dl_json=False, no_overlay=False):
    for index, snap in enumerate(organised_data):
        DL_MSG = f'Snap {index + 1}/{len(organised_data)} downloading...'

        filename = snap['location'] + ' - ' + \
            snap['create_time'] + ' - ' + snap['id']
        if dl_json:
            with open(f'{output_dir}/' + filename +
                      '.info.json', 'w') as json_file:
                json_file.write(json.dumps(snap, indent=2))
        if sys.platform == 'win32':
            cmd = [
                'aria2c.exe',
                snap['media']['raw_url'],
                '-d',
                output_dir,
                '-o'
            ]
        else:
            cmd = ['aria2c', snap['media']['raw_url'], '-d', output_dir, '-o']
        if snap['media']['raw_url'][-3:] == 'mp4':
            if os.path.exists(f'{cmd[-2]}/' + filename + '.mp4'):
                print(f'Snap {index + 1}/{len(organised_data)} '
                      'already downloaded.')
            else:
                print(DL_MSG + f' - {filename}.mp4')
                # Download snap without overlay
                if no_overlay:
                    subprocess.run(
                        cmd + [filename + '.mp4'], capture_output=True)
                else:
                    if snap['media']['video_overlay'] is not None:
                        merge_overlay = [
                            'ffmpeg',
                            "-y",
                            "-i",
                            snap['media']['raw_url'],
                            "-i",
                            snap['media']['video_overlay'],
                            "-filter_complex",
                            "[1][0]scale2ref[i][m];[m][i]overlay[v]",
                            "-map",
                            "[v]",
                            "-map",
                            "0:a?",
                            "-ac",
                            "2"
                        ]
                        # Merge video and overlay to one file using ffmpeg
                        subprocess.run(
                            merge_overlay + [f"{cmd[-2]}/{filename}.mp4"],
                            capture_output=True)
                        # Delete temp file
                    else:
                        subprocess.run(
                            cmd + [filename + '.mp4'], capture_output=True)

        else:
            if os.path.exists(f'{cmd[-2]}/' + filename + '.jpg'):
                print(f'Snap {index + 1}/{len(organised_data)} '
                      'already downloaded.')
            else:
                print(DL_MSG + f' - {filename}.jpg')
                subprocess.run(
                    cmd + [filename + '.jpg'], capture_output=True)



from typing import Dict, List
import requests
import random
import json
import pathlib
import sys


req_headers = {
    'User-Agent': random.choice(
        json.loads(open(f'{pathlib.Path(__file__).parent.absolute()}/utils/use'
                        'r-agents.json', 'r').read())),
    'Host': 'ms.sc-jpl.com',
    'Accept': '*/*',
    'Accept-Language': 'en-US',
    'Accept-Encoding': 'gzip, deflate, br',
    'Referer': 'https://map.snapchat.com/@0,0,7.67z',
    'Content-Type': 'application/json',
    'Origin': 'https://map.snapchat.com',
    'Content-Length': '2',
    'Connection': 'keep-alive',
    'DNT': '1',
    'Sec-GPC': '1',
    'TE': 'Trailers',
}


def get_epoch():
    for entry in json.loads(requests.post(
        'https://ms.sc-jpl.com/web/getLatestTileSet',
            headers=req_headers, json={}).text)['tileSetInfos']:
        if(entry['id']['type'] == 'HEAT'):
            return(entry['id']['epoch'])


def api_query(lat: float = 0, lon: float = 0.0, zl: int = 5, max_radius: int = 10000, mode: str = 'loc', snap_ids: list = []) -> Dict[str, List]:
    if mode == 'loc':
        available_snaps = []
        current_iteration = max_radius
        _epoch = get_epoch()
        try:
            print('Querying Snaps...')
            while current_iteration != 1:
                print(f'Querying with radius {current_iteration}...')
                payload = {
                    "requestGeoPoint": {
                        "lat": lat,
                        "lon": lon
                    },
                    "zoomLevel": zl,
                    "tileSetId": {
                        "flavor": "default",
                        "epoch": _epoch,
                        "type": 1
                    },
                    "radiusMeters": current_iteration,
                    "maximumFuzzRadius": 0
                }
                req_headers['Content-Length'] = str(len(str(payload)))
                api_data = requests.post('https://ms.sc-jpl.com/web/'
                                         'getPlaylist', headers=req_headers,
                                         json=payload).json()
                available_snaps = available_snaps + \
                    api_data['manifest']['elements']
                if(current_iteration > 2000):
                    current_iteration = current_iteration - 2000
                elif(current_iteration > 1000):
                    current_iteration = current_iteration - 100
                else:
                    current_iteration = 1
            print('Sorting list of Snaps...')
            return [i for n, i in enumerate(available_snaps)
                    if i not in available_snaps[n + 1:]]
        except Exception:
            sys.exit("You seem to have been rate limited, "
                     "please wait and try again.")

    if mode == 'snap':
        return requests.post(
            'https://ms.sc-jpl.com/web/getStoryElements',
            json={
                "snapIds": snap_ids
            }
        ).json()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snapmap_archiver', 'snapmap_archiver.utils']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'snapmap-archiver',
    'version': '1.3.0',
    'description': 'Download all Snapmaps content from a specific location.',
    'long_description': '# snapmap-archiver\n\nA tool written in Python to download all Snapmaps content from a specific location.\n\n![snapmap-archiver splash](/.github/img/Splash.png)\n\n## Setup\n\n`pip3 install snapmap-archiver`\n\n[View on PyPI](https://pypi.org/project/snapmap-archiver/)\n\nInstall dependencies with `pip3`.\n\n```sh\npip3 install -r requirements.txt\n```\n\n### Install [aria2c](http://aria2.github.io/)\n\nDownload `aria2c` from here:\n\n[https://aria2.github.io/](https://aria2.github.io/)\n\nThis is the downloader used for the fastest Snap download speeds.\n\n## Usage\n\n```sh\npython3 -m snapmap_archiver -o [OUTPUT DIR] -l="[LATITUDE],[LONGITUDE]"\n```\nUnfortunately you have to use the arbitrary `-l="lat,lon"` rather than just `-l "lat,lon"` when parsing negative numbers as `argsparse` interprets said numbers as extra arguments.\n\n### Optional Arguments\n\n#### Location\n\n`-l` is not required if an input file or Snap URL is provided. It can also be used multiple times to download Snaps from multiple locations in one command.\n\nE.g\n\n```sh\npython3 -m snapmap_archiver -o ~/Desktop/snap -l=\'123.123,123.123\' -l \'445.445,445.445\'\n```\n\n#### Input File\n\nWith `-t`, you can specify a file containing a list of line-separated Snap URLs or IDs.\n\nE.g\n\n```sh\npython3 -m snapmap_archiver -o ~/Desktop/snap -t ~/snaps.txt\n```\n\nInside `snaps.txt`:\n\n```\nhttps://map.snapchat.com/ttp/snap/Example/@-33.643495,115.741281,11.86z\nExample\nhttps://map.snapchat.com/ttp/snap/Example/\nhttps://map.snapchat.com/ttp/snap/Example/\n```\n\n#### Snap URL\n\nYou can also just pass 1 or more normal Snap URLs to the package to download it individually like this:\n\n```sh\npython3 -m snapmap_archiver -o ~/Desktop/snap \'https://map.snapchat.com/ttp/snap/Example/@-33.643495,115.741281,11.86z\'\n```\n\n#### Export JSON\n\nYou can export a JSON file with info about downloaded snaps with the `--write-json` argument, which will contain information like the time the Snap was posted, and the Snap location.\n\n#### Snap Radius\n\nThe radius from the coordinates you provide that will be included for downloads. `-r 20000` will download all Snaps within a 20km radius of your coordinates.\n\n#### No Overlay\n\nBy default the script merges the video and the overlay file into one file. With the `--no-overlay` argument you can disable this and only download the raw video.  \n',
    'author': 'king-millez',
    'author_email': 'Millez.Dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
